# Detect the distribution in use
%global __despace head -n 1 | tr -d '[:space:]' | sed -e 's/[(].*[)]//g'
%global __lower4 cut -c 1-4 | tr '[:upper:]' '[:lower:]'
%global __distfile %([ -f /etc/SuSE-release ] && echo /etc/SuSE-release || echo /etc/redhat-release)
%global __distinit %(sed -e 's/ release .*//' -e 's/\\([A-Za-z]\\)[^ ]*/\\1/g' %{__distfile} | %{__despace} | %{__lower4})
%global __distvers %(sed -e 's/.* release \\([^. ]*\\).*/\\1/' %{__distfile} | %{__despace})
# Identify CentOS Linux and Scientific Linux as rhel
%if "%{__distinit}" == "c" || "%{__distinit}" == "cl" || "%{__distinit}" == "sl" || "%{__distinit}" == "sls"
%global __distinit rhel
%endif
# Dist tag for Fedora is still "fc"
%if "%{__distinit}" == "f"
%global __distinit fc
%endif

# Fedora 10 onwards support noarch subpackages; by using one, we can
# put the arch-independent docs in a common subpackage and save lots
# of space on the mirrors
%if 0%{?fedora} > 9 || 0%{?rhel} > 5
%global noarch_docs_package 1
%else
%global noarch_docs_package 0
%endif

Name:		libssh2
Version:	1.4.0
Release:	1.0.cf.%{__distinit}%{__distvers}
Summary:	A library implementing the SSH2 protocol
Group:		System Environment/Libraries
License:	BSD
URL:		http://www.libssh2.org/
Source0:	http://libssh2.org/download/libssh2-%{version}.tar.gz
Patch0:		libssh2-1.2.9-utf8.patch
Patch1:		libssh2-1.4.0-c4a0e0.patch
BuildRoot:	%{_tmppath}/%{name}-%{version}-%{release}-root-%(id -nu)
BuildRequires:	openssl-devel
BuildRequires:	pkgconfig
BuildRequires:	zlib-devel
BuildRequires:	/bin/hostname
BuildRequires:	/usr/bin/man
# OpenSSH server used in test suite
# The SSH test fails on FC-2 and earlier distros because /etc/pam.d/sshd is only readable by
# root, causing non-root sshd sessions to be unable to authenticate (#64697)
%if 0%{?fedora} >= 3 || 0%{?rhel} >= 4
BuildRequires:	openssh-server
%endif
# We use matchpathcon to get the correct SELinux context for the ssh server
# initialization script so that it can transition correctly in an SELinux
# environment; matchpathcon is only available from FC-4 and moved from the
# libselinux to libselinux-utils package in F-10
%if 0%{?fedora} >= 4 || 0%{?rhel} >= 5
BuildRequires:	/usr/sbin/matchpathcon selinux-policy-targeted
%endif

%description
libssh2 is a library implementing the SSH2 protocol as defined by
Internet Drafts: SECSH-TRANS(22), SECSH-USERAUTH(25),
SECSH-CONNECTION(23), SECSH-ARCH(20), SECSH-FILEXFER(06)*,
SECSH-DHGEX(04), and SECSH-NUMBERS(10).

%package	devel
Summary:	Development files for libssh2
Group:		Development/Libraries
Requires:	%{name} = %{version}-%{release}
Requires:	pkgconfig

%description	devel
The libssh2-devel package contains libraries and header files for
developing applications that use libssh2.

%package	docs
Summary:	Documentation for libssh2
Group:		Development/Libraries
Requires:	%{name} = %{version}-%{release}
%if %{noarch_docs_package}
BuildArch:	noarch
%endif

%description	docs
The libssh2-docs package contains man pages and examples for
developing applications that use libssh2.

%prep
%setup -q

# Make sure things are UTF-8...
%patch0 -p1

# Fix undefined reference to _libssh_error in libgcrypt (upstream patch)
%patch1 -p1

# Make sshd transition appropriately if building in an SELinux environment
chcon $(/usr/sbin/matchpathcon -n /etc/rc.d/init.d/sshd) tests/ssh2.sh || :
chcon -R $(/usr/sbin/matchpathcon -n /etc) tests/etc || :
chcon $(/usr/sbin/matchpathcon -n /etc/ssh/ssh_host_key) tests/etc/{host,user} || :

%build
export CPPFLAGS="$(pkg-config --cflags openssl)"
%configure --disable-static --enable-shared
make %{?_smp_mflags}

%install
rm -rf %{buildroot}
make install DESTDIR=%{buildroot} INSTALL="install -p"
find %{buildroot} -name '*.la' -exec rm -f {} \;

# clean things up a bit for packaging
make -C example clean
rm -rf example/.deps
find example/ -type f '(' -name '*.am' -o -name '*.in' ')' -exec rm -v {} \;

# avoid multilib conflict on libssh2-devel
mv -v example/Makefile example/Makefile.%{_arch}

%check
# The SSH test will fail if we don't have /dev/tty, as is the case in some
# versions of mock (#672713)
if [ ! -c /dev/tty ]; then
	echo Skipping SSH test due to missing /dev/tty
	echo "exit 0" > tests/ssh2.sh
fi
# Apparently it fails in the sparc and arm buildsystems too
%ifarch %{sparc} %{arm}
echo Skipping SSH test on sparc/arm
echo "exit 0" > tests/ssh2.sh
%endif
make -C tests check

%clean
rm -rf %{buildroot}

%post -p /sbin/ldconfig

%postun -p /sbin/ldconfig

%files
%defattr(-,root,root,-)
%doc AUTHORS ChangeLog COPYING README NEWS
%{_libdir}/libssh2.so.1
%{_libdir}/libssh2.so.1.*

%files docs
%defattr(-,root,root,-)
%doc HACKING
%{_mandir}/man3/libssh2_*.3*

%files devel
%defattr(-,root,root,-)
%doc example/
%{_includedir}/libssh2.h
%{_includedir}/libssh2_publickey.h
%{_includedir}/libssh2_sftp.h
%{_libdir}/libssh2.so
%{_libdir}/pkgconfig/libssh2.pc

%changelog
* Wed Feb  1 2012 Paul Howarth <paul@city-fan.org> - 1.4.0-1.0.cf
- Update to 1.4.0
  - Added libssh2_session_supported_algs()
  - Added libssh2_session_banner_get()
  - Added libssh2_sftp_get_channel()
  - libssh2.h: bump the default window size to 256K
  - sftp-seek: clear EOF flag
  - userauth: provide more informations if ssh pub key extraction fails
  - ssh2_exec: skip error outputs for EAGAIN
  - LIBSSH2_SFTP_PACKET_MAXLEN: increase to 80000
  - knownhost_check(): don't dereference ext if NULL is passed
  - knownhost_add: avoid dereferencing uninitialized memory on error path
  - OpenSSL EVP: fix threaded use of structs
  - _libssh2_channel_read: react on errors from receive_window_adjust
  - sftp_read: cap the read ahead maximum amount
  - _libssh2_channel_read: fix non-blocking window adjusting 
- Add upstream patch fixing undefined function reference in libgcrypt backend
- BR: /usr/bin/man for test suite

* Mon Jan 16 2012 Paul Howarth <paul@city-fan.org> - 1.3.0-4.0.cf
- Skip the ssh test on sparc and arm buildsystems, where it tends to fail

* Fri Jan 13 2012 Paul Howarth <paul@city-fan.org> - 1.3.0-3.0.cf
- Example includes arch-specific bits, so move to devel package

* Wed Sep  7 2011 Paul Howarth <paul@city-fan.org> - 1.3.0-1.0.cf
- Update to 1.3.0
  - Added custom callbacks for performing low level socket I/O
  - sftp_read: advance offset correctly for buffered copies
  - libssh2_sftp_seek64: flush packetlist and buffered data
  - _libssh2_packet_add: adjust window size when truncating
  - sftp_read: a short read is not end of file

* Tue Aug 16 2011 Paul Howarth <paul@city-fan.org> - 1.2.9-1.0.cf
- Update to 1.2.9
  - Added libssh2_session_set_timeout() and libssh2_session_get_timeout() to
    make blocking calls get a timeout
  - configure and pkg-config: fix $VERSION
  - s/\.NF/.nf/ to fix wrong macro name caught by man --warnings
  - keepalive: add first basic man pages
  - sftp_write: flush the packetlist on error
  - sftp_write: clean offsets on error
  - msvcproj: added libs and debug stuff
  - SCP: fix incorrect error code
  - session_startup: init state properly
  - sftp_write_sliding: send the complete file
  - userauth_keyboard_interactive: skip code on zero length auth
  - _libssh2_wait_socket: fix timeouts for poll() uses
  - agent_list_identities: fix out of scope access
  - _libssh2_recv(): handle ENOENT error as EAGAIN
  - userauth_keyboard_interactive: fix buffer overflow
  - removed man pages for non-existing functions!
  - gettimeofday: fix name space pollution
  - _libssh2_channel_write: handle window_size == 0 better
- Drop upstreamed version patch
- Use patch rather than scripted iconv to fix character encoding
- Fix dist tag for CentOS 6 and Scientific Linux

* Wed Apr  6 2011 Paul Howarth <paul@city-fan.org> - 1.2.8-1.0.cf
- Update to 1.2.8
  - Added libssh2_free, libssh2_channel_get_exit_signal and
    libssh2_session_handshake
  - SFTP read/write remade and now MUCH faster, especially on high latency
    connections
  - Added new examples: ssh2_echo.c, sftp_append.c and sftp_write_sliding.c
  - userauth: derive publickey from private
  - Support unlimited number of host names in a single line of the
    known_hosts file
  - Fix memory leak in userauth_keyboard_interactive()
  - Fix memory leaks (two times cipher_data) for each sftp session
  - session_startup: manage server data before server identification
  - SCP: allow file names with bytes > 126
  - scp_recv: improved treatment of channel_read() returning zero
  - libssh2_userauth_authenticated: make it work as documented
  - variable size cleanup: match internal variable sizes better with the sizes
    of the fields used on the wire
  - channel_request_pty_size: fix reqPTY_state
  - sftp_symlink: return error if receive buffer too small
  - sftp_readdir: return error if buffer is too small
  - libssh2_knownhost_readfile.3: clarify return value
  - configure: stop using the deprecated AM_INIT_AUTOMAKE syntax
  - Fixed Win32 makefile which was now broken at resource build
  - kex_agree_hostkey: fix NULL pointer derefence
  - _libssh2_ntohu64: fix conversion from network bytes to uint64
  - ssize_t: proper typedef with MSVC compilers
  - zlib: Add debug tracing of zlib errors
  - decomp: increase decompression buffer sizes
- Skip the SSH test if we don't have /dev/tty (#672713)
- Nobody else likes macros for commands

* Wed Oct 13 2010 Paul Howarth <paul@city-fan.org> - 1.2.7-1.1.cf
- Improve support for running tests with SELinux in enforcing mode
- Avoid multilib conflict on libssh2-docs

* Wed Aug 18 2010 Paul Howarth <paul@city-fan.org> - 1.2.7-1.0.cf
- Update to 1.2.7
  - Better handling of invalid key files
  - inputchecks: make lots of API functions check for NULL pointers
  - libssh2_session_callback_set: extended the man page
  - SFTP: limit write() to not produce overly large packets
  - agent: make libssh2_agent_userauth() work blocking properly
  - _libssh2_userauth_publickey: reject method names longer than the data
  - channel_free: ignore problems with channel_close()
  - typedef: make ssize_t get typedef without LIBSSH2_WIN32
  - _libssh2_wait_socket: poll needs milliseconds
  - libssh2_wait_socket: reset error code to "leak" EAGAIN less
  - Added include for sys/select.h to get fd.set on some platforms
  - session_free: free more data to avoid memory leaks
  - openssl: make use of the EVP interface
  - Make libssh2_debug() create a correctly terminated string
  - userauth_hostbased_fromfile: packet length too short
  - handshake: Compression enabled at the wrong time
  - Don't overflow MD5 server hostkey

* Thu Jun 10 2010 Paul Howarth <paul@city-fan.org> - 1.2.6-1.0.cf
- Update to 1.2.6
  - Added libssh2_sftp_statvfs() and libssh2_sftp_fstatvfs()
  - Added libssh2_knownhost_checkp()
  - Added libssh2_scp_send64()
  - wait_socket made c89 compliant and use two fd_sets for select()
  - OpenSSL AES-128-CTR detection fixed
  - Proper keyboard-interactive user dialog in the sftp.c example
  - Fixed Build procedure for VMS
  - Fixed libssh2.dsw to use the generated libssh2.dsp
  - Several Windows-related build fixes
  - Fixed fail to init SFTP if session isn't already authenticated
  - Many tiny fixes that address clang-analyzer warnings
  - sftp_open: deal with short channel_write calls
  - libssh2_publickey_init: fixed to work better non-blocking
  - sftp_close_handle: add precation to not access NULL pointer
  - sftp_readdir: simplified and bugfixed
  - channel_write: if data has been sent, don't return EAGAIN
- Drop upstreamed AES-CTR detection patch

* Tue Jun  8 2010 Paul Howarth <paul@city-fan.org> - 1.2.5-3.1.cf
- RHEL-6 has noarch subpackages, so use one for docs

* Tue May 25 2010 Paul Howarth <paul@city-fan.org> - 1.2.5-3.0.cf
- Fix dist tag for RHEL-6 Beta

* Wed Apr 28 2010 Paul Howarth <paul@city-fan.org> - 1.2.5-2.0.cf
- Add buildreq openssh-server to enable additional test coverage
- Make sshd transition appropriately if building in an SELinux environment
  (a mock buildroot is not such an environment)
- Add buildreq /usr/sbin/matchpathcon to get appropriate SELinux context

* Wed Apr 14 2010 Paul Howarth <paul@city-fan.org> - 1.2.5-1.0.cf
- Update to 1.2.5
  - Add keep-alive support: libssh2_keepalive_config()/libssh2_keepalive_send()
  - Add libssh2_knownhost_addc(), libssh2_init() and libssh2_exit()
  - Add LIBSSH2_SFTP_S_IS***() macros
  - Fix memory leak in libssh2_session_startup()
  - Add missing error codes - shown as hangs in blocking mode
  - Fix memory leak in userauth_keyboard_interactive()
  - libssh2_knownhost_del: fix write to freed memory
  - Send and receive channel EOF before sending SSH_MSG_CHANNEL_CLOSE
  - Use AES-CTR from OpenSSL when available
  - Fixed gettimeofday to compile with Visual C++ 6
  - NULL dereference when window adjusting a non-existing channel
  - Avoid using poll on interix and mac os x systems
  - Fix scp memory leak
  - Correctly clear blocking flag after sending multipart packet
  - Reduce used window sizes by factor 10
  - libssh2_userauth_publickey_fromfile_ex() handles a NULL password
  - sftp_init() deal with _libssh2_channel_write() short returns
- Add patch to fix AES-CTR detection
- Add buildreq /bin/hostname for build host detection

* Mon Feb 15 2010 Paul Howarth <paul@city-fan.org> - 1.2.4-1.0.cf
- Update to 1.2.4
  - Resolve compile issues on Solaris x64 and UltraSPARC
  - Allow compiling with OpenSSL when AES isn't available
  - Fix Tru64 socklen_t compile issue with example/direct_tcpip.c

* Wed Feb  3 2010 Paul Howarth <paul@city-fan.org> - 1.2.3-1.0.cf
- Update to 1.2.3
  - Add libssh2_trace_sethandler()
  - Add the direct_tcpip.c example
  - Fix memory leak in userauth_publickey
  - Add support for authentication via SSH-Agent
  - Fix OpenSSH server keepalive (see NEWS)
- Drop padding patch

* Fri Jan 22 2010 Paul Howarth <paul@city-fan.org> - 1.2.2-5.0.cf
- Rename padding patch as per Fedora package
- sshd tests intentionally avoided (by not having openssh-server buildreq)
  because they appear to hang in the buildsystem

* Thu Dec 24 2009 Paul Howarth <paul@city-fan.org> - 1.2.2-2.0.cf
- Fix padding in ssh-dss signature blob encoding (#539444)

* Tue Nov 17 2009 Paul Howarth <paul@city-fan.org> - 1.2.2-1.0.cf
- Update to 1.2.2
  - fix crash when server sends an invalid SSH_MSG_IGNORE message
  - support for aes128-ctr, aes192-ctr, aes256-ctr ciphers as per RFC 4344
  - support for arcfour128 cipher as per RFC 4345
- Dist tag for Rawhide no longer needs special-casing
- Make docs package noarch where possible

* Wed Sep 30 2009 Paul Howarth <paul@city-fan.org> - 1.2.1-1.0.cf
- Update to 1.2.1 (many bugfixes - see NEWS)
- Drop upstreamed transport layer patch
- Devel package now includes a pkgconfig file (and requires pkgconfig)
- Don't redundantly include COPYING in docs and devel packages

* Wed Sep 23 2009 Paul Howarth <paul@city-fan.org> - 1.2-2.0.cf
- Update to 1.2 (see RELEASE-NOTES for details)
- Upstream source moved to libssh2.org
- Add upstream patch to fix transport layer bug causing invalid -39
  (LIBSSH2_ERROR_BAD_USE) errors

* Thu Apr  2 2009 Paul Howarth <paul@city-fan.org> - 1.1-1.0.cf
- Update to 1.1 (fixes memory leak and boosts sftp performance)

* Mon Jan 26 2009 Paul Howarth <paul@city-fan.org> - 1.0-1.0.cf
- Update to 1.0
- Help the configure script find openssl by setting CPPFLAGS

* Fri Aug  1 2008 Paul Howarth <paul@city-fan.org> - 0.18-9.0.cf
- Tweak dist tag macros to work on current Rawhide with three-part releasenum

* Fri Jul  4 2008 Paul Howarth <paul@city-fan.org> - 0.18-7.0.cf
- Import from Fedora
- Fix find syntax
- Add buildreq pkgconfig, missing dep from openssl-devel in older distributions

* Mon Feb 18 2008 Fedora Release Engineering <rel-eng@fedoraproject.org> - 0.18-7
- Autorebuild for GCC 4.3

* Wed Dec 05 2007 Chris Weyl <cweyl@alumni.drew.edu> 0.18-6
- rebuild for new openssl...

* Tue Nov 27 2007 Chris Weyl <cweyl@alumni.drew.edu> 0.18-5
- bump

* Tue Nov 27 2007 Chris Weyl <cweyl@alumni.drew.edu> 0.18-4
- add INSTALL arg to make install vs env. var

* Mon Nov 26 2007 Chris Weyl <cweyl@alumni.drew.edu> 0.18-3
- run tests; don't package test

* Sun Nov 18 2007 Chris Weyl <cweyl@alumni.drew.edu> 0.18-2
- split docs into -docs (they seemed... large.)

* Tue Nov 13 2007 Chris Weyl <cweyl@alumni.drew.edu> 0.18-1
- update to 0.18

* Sun Oct 14 2007 Chris Weyl <cweyl@alumni.drew.edu> 0.17-1
- update to 0.17
- many spec file changes

* Wed May 23 2007 Sindre Pedersen Bjørdal <foolish[AT]guezz.net> - 0.15-0.2.20070506
- Fix release tag
- Move manpages to -devel package
- Add Examples dir to -devel package

* Sun May 06 2007 Sindre Pedersen Bjørdal <foolish[AT]guezz.net> - 0.15-0.20070506.1
- Initial build
